/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __ARTICLE_LIST_H__
#define __ARTICLE_LIST_H__

#include <gdk/gdk.h>
#include <gtk/gtkstyle.h>
#include <gtk/gtkctree.h>

#include "article-db.h"
#include "server.h"
#include "group.h"
#include "pan-callback.h"
#include "status-item.h"

extern GdkColormap *cmap;
extern GdkColor read_color;
extern GdkColor unread_color;
extern GdkColor new_color;
extern GdkColor killfile_color;
extern GdkColor watched_color;

extern GtkStyle *unread_style;
extern GtkStyle *read_style;
extern GtkStyle *new_style;
extern GtkStyle *killfile_style;
extern GtkStyle *watched_style;

extern gboolean hide_mpart_child_nodes;

/****
*****  CALLBACKS
****/

/**
 * called whenever the article tree's active state filter changes.
 *
 * @param call_object the article ctree
 * @param call_arg an unsigned int of the state filter bitfields
 */
extern PanCallback* articlelist_state_filter_changed;

/**
 * called whenever the article tree's active group changes.
 *
 * @param call_object the article ctree
 * @param call_arg the new active group
 */
extern PanCallback* articlelist_group_changed;

/**
 * called whenever the article tree's sorting changes.
 *
 * @param call_object the article ctree
 * @param call_arg a GINT_TO_POINTER of the new sort type.
 */
extern PanCallback* articlelist_sort_changed;

/**
 * called whenever the article tree's threaded state changes.
 *
 * @param call_object the article ctree
 * @param call_arg NULL if threading is off, non-NULL if threading is on
 */
extern PanCallback* articlelist_thread_changed;

/**
 * called whenever the article tree's selection state changes.
 *
 * @param call_object the article ctree
 * @param call_arg adata of the selected article, or NULL if none
 */
extern PanCallback* articlelist_selection_changed;

/****
***** 
****/

/**
 * This pokes any given flag in the current state filter,
 * saves the state to disk,
 * and fires off the articlelist_state_filter_changed callbacks.
 */
void articlelist_poke_state_filter(guint flag, gboolean on);




/**
***
***
***
***
**/


/**
***  State Info
**/

Group* articlelist_get_current_group(void);

const Server* articlelist_get_current_server(void);

void articlelist_set_current_group (Server*, Group*);

void articlelist_set_sort_type (int sort_type);

void articlelist_set_threaded (gboolean threaded_on);

void articlelist_refresh (StatusItem*);

Article* articlelist_get_adata_from_node (GtkCTreeNode*);

GtkCTreeNode* articlelist_get_selected_node (void);

Article* articlelist_get_selected_adata (void);


/**
***  Stepping Through the Articles
**/

const char* articlelist_get_selected_message_id (void);

void articlelist_set_selected_message_id (const char* message_id);

void articlelist_set_selected_node (GtkCTreeNode*);

Article* articlelist_get_previous_article (void);

Article* articlelist_get_next_article (void);

GtkCTreeNode* articlelist_node_next (GtkCTreeNode*, gboolean sibling_only);

GtkCTreeNode* articlelist_node_prev (GtkCTreeNode*, gboolean sibling_only);

void articlelist_read_next_article (void);

void articlelist_read_next_unread_article (void);

void articlelist_read_next_thread (void);

void articlelist_read_prev_article (void);

void articlelist_read_prev_unread_article (void);

void articlelist_read_prev_thread (void);

/**
***  Contents
**/

void articlelist_load (Server  * server,
                       Group   * group,
                       StatusItem   * item);

/* this emits the "next step" signal between
   alist_total*2 and alist_total*3 times,
   depending on the killfile */
void articlelist_set_contents (Server   * sdata,
                               Group    * group,
                               Article ** articles,
                               int             article_qty,
                               StatusItem    * item);

void articlelist_clear (void);

void articlelist_queue_reload (void);

void articlelist_update_node (const Article* adata);

/**
***  Perform Operations on Articles
**/

void articlelist_select_all (void);

void articlelist_selected_decode (void);

void articlelist_selected_decode_as (void);

void articlelist_selected_delete (void);

void articlelist_selected_delete_thread (void);

void articlelist_selected_mark_read (void);

void articlelist_selected_mark_unread (void);

void articlelist_selected_open (void);

void articlelist_selected_thread_track (void);

void articlelist_selected_thread_download (void);

void articlelist_selected_thread_kill (void);

void articlelist_all_mark_read (void);

void articlelist_expire (article_db, Group *);

void articlelist_all_download (void);

void articlelist_selected_download (void);

/**
***  Misc
**/

GtkWidget* create_articlelist_ctree (void);

void articlelist_adata_read_changed (const Article*);


#endif /* __ARTICLE_LIST_H__ */
