/* Pan - User folders
 *
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>

#include <gnome.h>

#include "folder.h"
#include "folderlist.h"
#include "group.h"
#include "grouplist.h"
#include "globals.h"
#include "gui.h"


static void
create_folder (gchar *name)
{
	Group *gdata;

	gdata = group_new ();
	gdata->name = g_strdup (name);
	gdata->type = USER_FOLDER;

	g_ptr_array_add (folders, gdata);

	folderlist = g_slist_append (folderlist, gdata);

	folderlist_save ();
	grouplist_update_ui (grouplist_get_current_server());
}

static void
new_folder_cb (GtkWidget *dialog, int num, gpointer data)
{
	gchar *pch = gtk_editable_get_chars (GTK_EDITABLE (data), 0, -1);

	switch (num) {
	case 0: /* OK Button */
		create_folder (pch);
		break;
	case 1: /* Cancel Button */
		break;
	default: break;
	}

	gtk_widget_destroy (GTK_WIDGET (dialog));
	g_free (pch);
}

/**
 * new_folder:
 *
 * User has clicked on "New Folder...", pop up a dialog for the folder name,
 * grab the name, create the backend for it.
 **/
void
new_folder_dialog (void)
{
	GtkWidget *label;
	GtkWidget *hbox;
	GtkWidget *entry = gtk_entry_new ();
	GnomeDialog *dialog = (GnomeDialog *) gnome_dialog_new (_("New Folder"),
								GNOME_STOCK_BUTTON_OK,
								GNOME_STOCK_BUTTON_CANCEL,
								NULL);

	label = gtk_label_new (_("Name of new folder:"));

	hbox = gtk_hbox_new (FALSE, 0);

	gtk_container_add (GTK_CONTAINER (dialog->vbox), hbox);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, GNOME_PAD);
	gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);

	gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
			    GTK_SIGNAL_FUNC (new_folder_cb),
			    entry);

	gui_popup_draw (GTK_WIDGET (dialog), Pan.window);
}

#if 0
static void
rename_folder (Group *group, gchar *newname)
{

}
#endif

static void
rename_folder_cb (GtkWidget *dialog, int num, gpointer data)
{
	gchar *pch = gtk_editable_get_chars (GTK_EDITABLE (data), 0, -1);
//	gchar *group = gtk_object_get_data (GTK_OBJECT (dialog), "gdata");


	if (num == 1) /* Cancel Button */
	{
		
	}
	else if (num == 0) /* OK Button */
	{
//		rename_folder (group, pch);
	}

	gtk_widget_destroy (GTK_WIDGET (dialog));
	g_free (pch);
}

/**
 * folder_rename_dialog:
 *
 **/
void
rename_folder_dialog (Group *gdata)
{
	GtkWidget *label;
	GtkWidget *hbox;
	GtkWidget *entry;
	GnomeDialog *dialog;


	/* only allow renaminig of user folders, maybe groups later */
	if (gdata->type != USER_FOLDER)
		return;

	entry = gtk_entry_new ();
	dialog = (GnomeDialog *) gnome_dialog_new (_("Rename Folder"),
								GNOME_STOCK_BUTTON_OK,
								GNOME_STOCK_BUTTON_CANCEL,
								NULL);

	label = gtk_label_new (_("Name of folder:"));

	if (gdata->name)
		gtk_entry_set_text (GTK_ENTRY (entry), gdata->name);

	hbox = gtk_hbox_new (FALSE, 0);

	gtk_container_add (GTK_CONTAINER (dialog->vbox), hbox);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, GNOME_PAD);;
	gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);

	gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
			    GTK_SIGNAL_FUNC (rename_folder_cb),
			    entry);

	gui_popup_draw (GTK_WIDGET (dialog), Pan.window);
}
