/* Pan - User folders
 *
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>

#include <gnome.h>

#include "folderlist.h"
#include "globals.h"
#include "group.h"
#include "grouplist.h"
#include "pan-db.h"
#include "prefs.h"
#include "util.h"

GSList       *folderlist;
GPtrArray    *folders;
Server  *folder_sdata;

static void
folderlist_load_folder (const char *key, const char *val, gpointer data)
{
	Group *gdata = NULL;
	gchar **sd = g_strsplit (val, "\n", -1);
	gboolean end =  FALSE;

	gdata = group_new ();
	gdata->name = g_strdup (key);
	gdata->type = USER_FOLDER;
	gdata->flags = end ? 0 : atoi (sd[0]);
	gdata->description = !end && sd[1] && *sd[1] ? strdup(sd[1]) : NULL;

	g_strfreev (sd);
	g_ptr_array_add (folders, gdata);

	folderlist = g_slist_append (folderlist, gdata);
}

static void
folderlist_load_thread (void)
{
	gchar *path;
	pan_db db;

	path = g_strdup_printf ("/%s/folderlist.db", data_dir);
	db = pan_db_open (path);
	g_free (path);

	/* load the folders */
	if (db != NULL) {
		pan_db_foreach (db, folderlist_load_folder, NULL);
		pan_db_close (db);
	}

//	grouplist_update_ui (grouplist_get_current_server());
}

void
folderlist_load (void)
{
	pthread_t thread;
	pthread_create (&thread, NULL, (void*)folderlist_load_thread, NULL);
	pthread_detach (thread);
}

void
folderlist_init (void)
{
	gchar *dbpath;

	folders = g_ptr_array_new();

	folder_sdata = (Server*) g_new0 (Server, 1);
	folder_sdata->name   = g_strdup ("folders");
	folder_sdata->groups = folders;
	if (folder_sdata->db == NULL)
	{
		gchar *path = g_strdup_printf ("/%s/folders.db", data_dir);
		folder_sdata->db = pan_db_open (path);
		g_free (path);
	}

	dbpath = g_strdup_printf ("/%s/%s/", data_dir, folder_sdata->name);
	directory_check (dbpath);
	g_free (dbpath);
}

void
folderlist_shutdown (void)
{
	pan_db_close (folder_sdata->db);
}

#if 0
void
folderlist_save_folder (Group *gdata)
{
	gchar *pch = NULL;
	pan_db db = 0;

	pch = g_strdup_printf ("/%s/folderlist.db", data_dir);
	db = pan_db_open (pch);
	g_free (pch);
	if (!db)
		return;

	pch = g_strdup_printf ("%d\n%s\n",
			       gdata->flags,
			       gdata->description?gdata->description:"");
	pan_db_put_value_str (db, gdata->name, pch);
	g_free (pch);

	pan_db_close (db);
}
#endif

void
folderlist_save (void)
{
	gchar *path;
	pan_db db;
	int i;
	Group **pfolders = (Group**)folders->pdata;

	path = g_strdup_printf ("/%s/folderlist.db", data_dir);
	db = pan_db_open (path);
	g_free (path);

	if (!db)
		return;

	for (i=0; i<folders->len; ++i)
	{
		const Group *gdata = pfolders[i];
		gchar *pch = g_strdup_printf ("%d\n%s\n",
					      gdata->flags,
					      gdata->description?gdata->description:"");
		pan_db_put_value_str (db, gdata->name, pch);
		g_free (pch);
	}

	pan_db_close (db);
}

#if 0
static void
folderlist_picker_clicked_cb (GnomeDialog *dialog,
			      int          button,
			      gpointer     user_data)
{

	pan_lock ();
	gtk_widget_destroy (GTK_WIDGET (dialog));
	pan_unlock ();
}

static void
folderlist_picker_select_row_cb (GtkCList *clist,
				 gint row,
				 gint column,
				 GdkEvent *event,
				 gpointer user_data)
{
	GtkWidget *dialog = user_data;
	Group *folder = gtk_clist_get_row_data (GTK_CLIST (clist),
						     row);
	gtk_object_set_data (GTK_OBJECT (dialog), "selectedfolder", folder);
}

GtkWidget *
folderlist_picker_new (void)
{
	GtkWidget *clist;
	GtkWidget *dialog;
	GSList *l =  NULL;

	dialog = gnome_dialog_new (_("Pick Folder"),
				   GNOME_STOCK_BUTTON_OK,
				   GNOME_STOCK_BUTTON_CANCEL,
				   NULL);

	clist = gtk_clist_new(1);
	gtk_clist_column_titles_passive (GTK_CLIST (clist));
	gtk_clist_set_selection_mode (GTK_CLIST (clist),
				      GTK_SELECTION_SINGLE);

	for (l=folderlist; l!=NULL; l=l->next)
	{
		Group *folder = (Group*) l->data;
		gchar *text[2];
		int row;

		if (!folder)
			continue;

		text[0] = folder->name;
		text[1] = NULL;
		row = gtk_clist_prepend (GTK_CLIST (clist), text);

		gtk_clist_set_row_data (GTK_CLIST (clist), row, folder);
	}

	gtk_container_add (GTK_CONTAINER (GNOME_DIALOG (dialog)->vbox), clist);

	gtk_signal_connect (GTK_OBJECT (clist), "select_row",
			    GTK_SIGNAL_FUNC (folderlist_picker_select_row_cb),
			    dialog);

	return dialog;
}
#endif
