/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __GROUPS_H__
#define __GROUPS_H__

#include <glib.h>
#include "server.h"

enum {
        STATE_FILTER_UNREAD                     = (1<<0),
        STATE_FILTER_READ                       = (1<<1),
        STATE_FILTER_WATCHED                    = (1<<2),
        STATE_FILTER_KILLFILE                   = (1<<3),
        STATE_FILTER_NORMAL_RANK                = (1<<4),
        STATE_FILTER_COMPLETE_BINARIES          = (1<<5),
        STATE_FILTER_INCOMPLETE_BINARIES        = (1<<6),
        STATE_FILTER_NONBINARIES                = (1<<7),
        STATE_FILTER_SAVED                      = (1<<8),
        STATE_FILTER_QUEUED                     = (1<<9),
        STATE_FILTER_IDLE                       = (1<<10)
};

typedef enum {
	GROUP,
	USER_FOLDER	
} GroupType;

typedef struct
{
	gchar*    name;		/* alt.what.is.my.name */
	gchar*    description;	/* xgtitle */
	GroupType type;
	gint      flags;		/* new, subscribed */
	guint	  state_filter;	/* for filtering articles */
}
Group;

extern int compare_Group_std (const void*, const void*);
extern gint compare_Group (Group *, Group *);
extern void check_new_group_list_for_newness (Group** groups, int qty);
extern void choose_download_dialog (const Server*, Group *);
extern void group_free			(Group *gdata);
extern void get_all_groups		(void);
extern void get_new_groups		(void);
extern void group_empty_dialog (Group *gdata);
extern void group_unload (Group *gdata);
extern Group* group_copy (const Group *src);
extern Group* group_new (void);

//=============

extern void add_group_to_clist (Server *sdata, Group *gdata);
extern void add_group_to_clist_if_match(Server *sdata, Group*, gint flag_mask); 
extern void add_group_to_clist_if_match_gfunc(Server *sdata, gpointer, gpointer);

//=============

extern int group_get_attrib_i	(const Server*, const Group *, const char *);
extern void group_set_attrib_i	(const Server*, const Group *, const char *, int);
extern void group_add_attrib_i (const Server*, const Group *, const char *, int);
extern void group_set_attrib_string (const Server*, const Group *gdata, const char *attrib, const char *value);
extern gchar* group_get_attrib_string (const Server*, const Group *gdata, const char *attrib);

//=============

extern void group_download_all_articles (Server*, Group*);

extern void group_remove_dialog (Group *gdata);


#endif /* __GROUPS_H__ */
