/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*********************
**********************  Includes
*********************/

#include <config.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-i18n.h>
#include <libgnomeui/libgnomeui.h>

#include "acache.h"
#include "globals.h"
#include "gui.h"
#include "log.h"
#include "prefs.h" /* for download_dir */
#include "queue.h"
#include "task-decode.h"
#include "save.h"
#include "util.h"

/*********************
**********************  Defines / Enumerated types
*********************/

/*********************
**********************  Macros
*********************/

/*********************
**********************  Structures / Typedefs
*********************/

/*********************
**********************  Private Function Prototypes
*********************/

static void save_file_cb (GtkWidget* entry, gpointer user_data);

/*********************
**********************  Variables
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

/*********************
**********************  BEGINNING OF SOURCE
*********************/

/************
*************  PUBLIC ROUTINES
************/

void save_current_article_cb(GtkWidget* w, gpointer p)
{
	save_current_article();
}

void save_current_article(void)
{
	if (Pan.active_adata)
		save_article (Pan.active_adata->message_id);
}

void save_article (const gchar* message_id)
{
	GtkWidget* gfs = NULL;
	gchar* head = NULL;
	gchar* body = NULL;

	g_return_if_fail (message_id!=NULL);
	g_return_if_fail (*message_id!='\0');

	/* if we don't have the article already, download it */
        if ( !acache_body_exists(message_id)
	     || !acache_header_exists(message_id))
	{
		pan_error_dialog ( "Must load the article first." );
		return;
	}

	head = acache_load_header (message_id);
	body = acache_load_body (message_id);
	gfs = gtk_file_selection_new ("Save As...");
	gtk_object_set_data_full (GTK_OBJECT(gfs), "body", body, g_free);
	gtk_object_set_data_full (GTK_OBJECT(gfs), "head", head, g_free);

	gtk_signal_connect (
		GTK_OBJECT(GTK_FILE_SELECTION(gfs)->ok_button),
		"clicked", save_file_cb, (gpointer)gfs );
	gtk_signal_connect_object (
		GTK_OBJECT(GTK_FILE_SELECTION(gfs)->cancel_button),
		"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
		(gpointer)gfs);
		            
	gtk_widget_show(gfs);
}

/************
*************  PRIVATE ROUTINES
************/

static void
real_save_file (GtkFileSelection *gfs)
{
	FILE *fp;
	gchar* body = gtk_object_get_data (GTK_OBJECT(gfs), "body");
	gchar* head = gtk_object_get_data (GTK_OBJECT(gfs), "head");
	gchar* filename = gtk_object_get_data (GTK_OBJECT(gfs), "filename");
      
	/* open the file... */	
	fp = fopen (filename, "w+");
	if (!fp) {
		pan_error_dialog (_("Unable to create file ``%s''"), filename);
		return;
	}

	/* write the file... */
	fputs(head, fp);
	fputs("\n\n", fp);
	fputs(body, fp);
	fclose(fp);

	log_add_va ("Saved file ``%s''", filename);
	gtk_widget_destroy (GTK_WIDGET(gfs));
}

static void
yes_im_sure_cb(gint reply, gpointer data)
{
	if (reply==0) /* yes */
		real_save_file (GTK_FILE_SELECTION(data));
}

/*****
******
*****/

static void
save_file_cb(GtkWidget* entry, gpointer user_data)
{
	GtkFileSelection *gfs = GTK_FILE_SELECTION(user_data);
	gchar *filename = gtk_file_selection_get_filename(gfs);
	if (filename && *filename)
	{
		gtk_object_set_data(GTK_OBJECT(gfs),"filename",filename);
		if (file_exists(filename))
		{
			GtkWidget *w = NULL;
			w = gnome_question_dialog_parented (
				_("File Exists.  Overwrite this file?"),
				yes_im_sure_cb, gfs,
				GTK_WINDOW(Pan.window) );
		}
		else
		{
			real_save_file (gfs);
		}
	}
}

/*****
******
*****/

typedef struct
{
	GtkWidget *default_dir_rb;
	GtkWidget *group_default_dir_rb;
	GtkWidget *user_specified_rb;
	GtkWidget *user_specified_entry;
	GtkWidget *encoded_filename_rb;
	GtkWidget *subject_filename_rb;
	GtkWidget *user_defined_filename_rb;
	GtkWidget *append_groupname_tb;

	Server *sdata;
	Group  *gdata;

	GSList *gslists_of_unowned_adatas;
}
SaveAsDialogStruct;

static gchar *last_save_as_path = NULL;

static gboolean append_groupname = FALSE;

static void
save_attachment_as_dialog_clicked_cb (
	GnomeDialog* dialog,
	int button,
	gpointer user_data)
{
	SaveAsDialogStruct* save = (SaveAsDialogStruct*)user_data;
	GSList * l;
	GSList * filenames = NULL;
	GSList * filename = NULL;
	gchar *path;

	/* cancel button */
	if (button==1)
	{
		pan_lock();
		gtk_widget_destroy (GTK_WIDGET(dialog));
		pan_unlock();
		return;
	}

	/* get path */
	path = NULL;
	if (gtk_toggle_button_get_active( /* user-specified directory */
			GTK_TOGGLE_BUTTON(save->user_specified_rb)))
		path = gtk_editable_get_chars (
			GTK_EDITABLE(save->user_specified_entry), 0, -1);
	else if (gtk_toggle_button_get_active( /* group preferences */
			GTK_TOGGLE_BUTTON(save->group_default_dir_rb)))
		path = group_get_attrib_string (
			save->sdata, save->gdata, "download_dir");
	else path = g_strdup (download_dir); /* global preferences */

	/* append the groupname, if desired */
	if (gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(save->append_groupname_tb)))
	{
		gchar * pch = g_strdup_printf ("%s/%s",
					       path,
					       save->gdata->name);
		g_free (path);
		path = pch;
	}

	/* get filenames */
	if (gtk_toggle_button_get_active( /* use encoded filename */
			GTK_TOGGLE_BUTTON(save->encoded_filename_rb)))
	{
		for (l=save->gslists_of_unowned_adatas; l!=NULL; l=l->next)
			filenames = g_slist_append(filenames, NULL);
	}
	else if (gtk_toggle_button_get_active( /* use subject as filename */
			GTK_TOGGLE_BUTTON(save->subject_filename_rb)))
	{
		for (l=save->gslists_of_unowned_adatas; l!=NULL; l=l->next)
		{
			/* use the subject of the first article in the list */
			GSList * article_parts = (GSList*)l->data;
			const Article* adata = ARTICLE(article_parts->data);
			char* name = g_strdup (adata->subject);
			filenames = g_slist_append (filenames, name);
		}
	}
	else /* user-specified filename */
	{
		/* FIXME */
		g_assert (FALSE);
	}

	/* queue 'em */
	filename = filenames;
	for (l=save->gslists_of_unowned_adatas; l!=NULL; l=l->next)
	{
		GSList *l2 = (GSList*) l->data;
		GSList *tmp;

		/* mark these articles as queued */
		for (tmp=l2; tmp!=NULL; tmp=tmp->next)
		{
			article_add_flag (
				save->sdata,
				save->gdata,
				ARTICLE(tmp->data),
				STATE_DECODE_QUEUED,
				tmp->next==NULL);
		}

		/* add to queue */
		task_decode_validate_and_queue_or_self_destruct (
			TASK_DECODE(task_decode_new(
				save->sdata,
				save->gdata,
				ARTICLE(l2->data)->subject,
				path,
				(const char*)(filename->data),
				l2,
				FALSE)));

		filename = filename->next;
	}

	/* cleanup */
	g_free (path);
	g_slist_foreach (filenames, (GFunc)g_free, NULL);
	g_slist_free (filenames);
	pan_lock();
	gtk_widget_destroy (GTK_WIDGET(dialog));
	pan_unlock();
}


static void
append_groupname_toggled_cb (
	GtkToggleButton *togglebutton,
	gpointer user_data)
{
	append_groupname = gtk_toggle_button_get_active(togglebutton);
}
static void
last_save_as_path_rb_toggled_cb (
	GtkToggleButton *togglebutton,
	gpointer user_data)
{
	SaveAsDialogStruct *save = (SaveAsDialogStruct*) user_data;
	gtk_widget_set_sensitive (
		GTK_WIDGET(save->user_specified_entry),
		gtk_toggle_button_get_active(togglebutton));
}

static void
save_attachment_as_dialog_destroy_cb (
	GtkObject* o,
	gpointer user_data)
{
	SaveAsDialogStruct *save = (SaveAsDialogStruct*) user_data;
	GSList* l;

	/* free the save-as-dialog utility structure */
	for (l=save->gslists_of_unowned_adatas; l!=NULL; l=l->next)
	{
		GSList* l2 = (GSList*) l->data;
		/*g_slist_foreach (l2, (GFunc)article_free, NULL);*/
		g_slist_free (l2);
	}
	g_slist_free (save->gslists_of_unowned_adatas);
	g_free (save);
}

static void
user_specified_path_changed_cb (
	GtkEditable *editable,
	gpointer user_data)
{
	g_free (last_save_as_path);
	last_save_as_path = gtk_editable_get_chars (
		GTK_EDITABLE(editable), 0, -1);
}

static void
create_save_attachment_as_dialog (
	Server * server,
	Group * group,
	GSList * gslists_of_unowned_adatas)
{
	SaveAsDialogStruct* data = g_new0 (SaveAsDialogStruct, 1);
	GtkWidget* dialog = NULL;
	GtkWidget* frame = NULL;
	GtkWidget* entry = NULL;
	GtkWidget* hbox = NULL;
	GtkWidget* vbox = NULL;
	GtkWidget* w = NULL;
	gchar *pch = NULL;

	data->sdata = server;
	data->gdata = group;
	data->gslists_of_unowned_adatas = gslists_of_unowned_adatas;

	/* create & parent the dialog */
	dialog = gnome_dialog_new (_("Save Attachments As"),
				   GNOME_STOCK_BUTTON_OK,
				   GNOME_STOCK_BUTTON_CANCEL,
				   NULL);
	gnome_dialog_set_default (GNOME_DIALOG(dialog), 1); /* cancel */

	/* "Location" pane */
	frame = gtk_frame_new (_("Location"));
	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
	vbox = gtk_vbox_new (FALSE, 0);

	/* global default directory */
	w = gtk_radio_button_new_with_label (NULL, download_dir);
	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
	data->default_dir_rb = w;

	/* group default directory */
	pch = group_get_attrib_string (server, group, "download_dir");
	if (pch != NULL) {
		w = gtk_radio_button_new_with_label_from_widget (
			GTK_RADIO_BUTTON(w), pch);
		gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
		data->group_default_dir_rb = w;
		g_free (pch);
	}

	/* user-specified directory */
	hbox = gtk_hbox_new (FALSE, 0);
	w = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON(w));
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
	data->user_specified_rb = w;
	entry = gtk_entry_new ();
	gtk_widget_set_usize (GTK_WIDGET(entry), 400, 0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w),
				      last_save_as_path!=NULL);
	gtk_widget_set_sensitive (GTK_WIDGET(entry),
				  last_save_as_path!=NULL);
        if (!last_save_as_path)
		last_save_as_path = g_strdup (download_dir);
	gtk_entry_set_text (GTK_ENTRY(entry), last_save_as_path);
	gtk_box_pack_start (GTK_BOX(hbox), entry, TRUE, TRUE, 0);
	data->user_specified_entry = entry;
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);


	/* append group name to the path? */
	pch = g_strdup_printf (_("Add ``/%s'' to path"), group->name);
	w = gtk_check_button_new_with_label (pch);
	g_free (pch);
	data->append_groupname_tb = w;
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), append_groupname);
	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, GNOME_PAD_SMALL);

	/* add "Location" to the dialog */
	gtk_container_add (GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start (
		GTK_BOX (GNOME_DIALOG (dialog)->vbox), frame, TRUE, TRUE, 0);




	/* "Filename" pane */
	frame = gtk_frame_new (_("Filename"));
	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
	vbox = gtk_vbox_new (TRUE, 0);

	/* use encoded filename */
	w = gtk_radio_button_new_with_label (NULL, _("Use Encoded Filename"));
	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
	data->encoded_filename_rb = w;

	/* use subject */
	w = gtk_radio_button_new_with_label_from_widget (
		GTK_RADIO_BUTTON(w), _("Use Article Subject"));
	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
	data->subject_filename_rb = w;

	/* user-specified filenames */
	/* FIXME */

	/* add "Filename" to the dialog */
	gtk_container_add (GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start (
		GTK_BOX (GNOME_DIALOG (dialog)->vbox), frame, TRUE, TRUE, 0);


	/* prep the dialog */
	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
		GTK_SIGNAL_FUNC(save_attachment_as_dialog_clicked_cb),
		data);
	gtk_signal_connect (GTK_OBJECT(dialog), "destroy",
		GTK_SIGNAL_FUNC(save_attachment_as_dialog_destroy_cb),
		data); 
	gtk_signal_connect (GTK_OBJECT(data->user_specified_rb), "toggled",
		last_save_as_path_rb_toggled_cb, data);
	gtk_signal_connect (GTK_OBJECT(data->append_groupname_tb), "toggled",
		append_groupname_toggled_cb, data);
	gtk_signal_connect (
		GTK_OBJECT(entry), "changed",
		user_specified_path_changed_cb, NULL);

	gui_popup_draw (dialog, Pan.window);
}



void
save_attachment_as (
	Server* server,
	Group* group,
	GSList* gslists_of_unowned_adatas)
{
	create_save_attachment_as_dialog (
           server, group, gslists_of_unowned_adatas);
}

