/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __SERVER_H__
#define __SERVER_H__

#include <glib.h>
#include "pan-db.h"

typedef struct
{
	gchar    * address;                   /* full address, i.e. news.erols.com */
	gchar    * name;	              /* pretty name, i.e. Erols */
	gchar    * username;                  /* auth username */
	gchar    * password;                  /* auth password */
	gboolean   posting : 1;               /* is posting allowed? */
	gboolean   gen_msgid : 1;
	gboolean   need_auth : 1;             /* need authorization ? */
	int        port;	              /* port #, NNTP usually 119, SMTP usually 25 */
	int        max_connections;           /* max socket connections this server allows */
        int        idle_secs_before_timeout;
	GPtrArray *groups;
	struct PanDB *db;
}
Server;

#define SERVER(a) ((Server*)(a))

extern void       server_menu_update   (void);
extern void       server_init          (void);
extern void       server_shutdown      (void);
extern GSList*    server_list_load     (void);
extern void       server_free          (Server *);
extern void*      server_get_attrib_db (void);
extern Server* server_fetch       (const gchar *name);
extern gboolean   server_has_group     (const Server*, const char* groupname);


#endif /* __SERVER_H__ */
