/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*********************
**********************  Includes
*********************/

#include <config.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-i18n.h>
#include <libgnomeui/libgnomeui.h>

#include "prefs.h"
#include "queue.h"
#include "status-item-view.h"
#include "task-manager.h"
#include "util.h"

#include "xpm/arrow_up.xpm"
#include "xpm/arrow_down.xpm"
#include "xpm/traffic_green.xpm"
#include "xpm/traffic_red.xpm"
#include "xpm/delete_row.xpm"
#include "xpm/delete_sheet.xpm"
#include "xpm/requeue.xpm"
#if 0 /* not ready yet; wait for next version */
#include "xpm/queue_load.xpm"
#include "xpm/queue_save.xpm"
#endif
#include "xpm/x.xpm"
#include "xpm/server.xpm" /*TODO*/

/*********************
**********************  Defines / Enumerated types
*********************/

/*********************
**********************  Macros
*********************/

/*********************
**********************  Structures / Typedefs
*********************/

typedef struct
{
	GtkWidget *queue_clist;
	GtkWidget *run_vbox;
	GtkWidget *window;
	GtkWidget *top;
	GtkWidget *socket_lb;
	GtkWidget *max_connections_sb;
	GtkWidget *dedicate_read_socket_tb;
	GtkWidget *online_button;
	GtkWidget *fail_clist;
	GtkWidget *max_tries_sb;

	GHashTable *task_to_run_widget;
	guint timeout_id;
	gboolean dampen_move_feedback_loop;
	gboolean dampen_connection_feedback_loop;

}
ManagerUI;

/*********************
**********************  Private Function Prototypes
*********************/

static void running_task_cancel_cb (GtkWidget*, gpointer);

/*********************
**********************  Constants
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

/*********************
**********************  Variables
*********************/

/***********
************  Extern
***********/

extern GtkTooltips *ttips;

/***********
************  Public
***********/

/***********
************  Private
***********/

static ManagerUI* manager_ui = NULL;

static GnomeUIInfo running_task_popup[] =
{
	{
		GNOME_APP_UI_ITEM,
		N_("Cancel Task"),
		N_("Cancel this running Task"),
		running_task_cancel_cb
	},
	/* FIXME: add speed limit */
	GNOMEUIINFO_END
};

static GtkWidget *running_task_popup_menu = 0;

/*********************
**********************  BEGINNING OF SOURCE
*********************/

/************
*************  PUBLIC ROUTINES
************/

static int
tm_index (int abs_index, QueueTaskStatus status)
{
	int i;
	int index = 0;
	const GSList* l = NULL;

	if (abs_index == -1)
		return abs_index;

	for (i=0, l=queue_get_tasks(); l!=NULL && i!=abs_index; ++i, l=l->next)
	{
		Task* task = TASK(l->data);
		QueueTaskStatus qts = queue_get_task_status (task);
		if (qts == status)
			++index;
	}

	return index;
}

static void
add_task_to_fail_clist (ManagerUI* ui, Task* task, int index)
{
	int row;
	gchar* text [] = {
		status_item_describe (STATUS_ITEM(task)),
		g_strdup (STATUS_ITEM(task)->last_error)
	};

	pan_lock ();
	row = gtk_clist_insert (GTK_CLIST(ui->fail_clist), index, text);
	gtk_clist_set_row_data (GTK_CLIST(ui->fail_clist), row, task);
	pan_unlock ();

	g_free (text[0]);
	g_free (text[1]);
}

static const char*
status_to_string (QueueTaskStatus status)
{
	switch (status) {
		case QUEUE_TASK_STATUS_NOT_QUEUED: return _("Not Queued");
		case QUEUE_TASK_STATUS_QUEUED: return _("Queued");
		case QUEUE_TASK_STATUS_RUNNING: return _("Running");
		case QUEUE_TASK_STATUS_FAILED: return _("Failed");
		case QUEUE_TASK_STATUS_ABORTING: return _("Aborting");
		case QUEUE_TASK_STATUS_CONNECTING: return _("Connecting");
		default: return _("Unknown");
	}
	/*NOTREACHED*/
}


static void
add_task_to_queue_clist (ManagerUI* ui, Task* task, int index)
{
	int row;
	char trybuf[8];
	char* text [] = {
		trybuf,
		(char*)status_to_string(queue_get_task_status (task)),
		status_item_describe (STATUS_ITEM(task))
	};
	sprintf (trybuf, "%d", task->tries);

	pan_lock ();
	row = gtk_clist_insert (GTK_CLIST(ui->queue_clist), index, text);
	gtk_clist_set_row_data (GTK_CLIST(ui->queue_clist), row, task);
	pan_unlock ();

	g_free (text[2]);
}

static void
running_task_cancel_cb (GtkWidget* w, gpointer user_data)
{
	queue_task_abort (TASK(STATUS_ITEM_VIEW(user_data)->item));
}

static void
add_task_to_run_clist (ManagerUI* ui, Task* task, int index)
{
	GtkWidget * view = NULL;
	GtkWidget * frame = NULL;
	GtkWidget * eventbox = NULL;

	/* setup */
	pan_lock ();
	
	eventbox = gtk_event_box_new();

	/* view / separator */
	view = status_item_view_new_with_item (
		SIV_STYLE_FULL, STATUS_ITEM(task));
	if (task->sock) {
		status_item_view_set_socket (STATUS_ITEM_VIEW(view),task->sock);
		gnome_popup_menu_attach (running_task_popup_menu, eventbox, view);
	}
	gtk_container_add (GTK_CONTAINER(eventbox), view);
	frame = gtk_frame_new (NULL);
	gtk_container_set_border_width (GTK_CONTAINER(view), GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER(frame), GNOME_PAD);
	gtk_container_add (GTK_CONTAINER(frame), eventbox);

	/* rearrange */
	gtk_box_pack_start (GTK_BOX(ui->run_vbox), frame, 0, 0, 0);
	gtk_box_reorder_child (GTK_BOX(ui->run_vbox), frame, index);

	/* cleanup */
	gtk_widget_show_all (GTK_WIDGET(ui->run_vbox));
	pan_unlock();
}

static void
remove_task (ManagerUI* ui, Task* task)
{
	int i;
	gboolean done;

	pan_lock();

	done = FALSE;

	/* check the queue list */
	if (!done) {
		i = gtk_clist_find_row_from_data (
			GTK_CLIST(ui->queue_clist), task);
		if (i!=-1) {
			gtk_clist_remove (GTK_CLIST(ui->queue_clist), i);
			done = TRUE;
		}
	}

	/* check the run list */
	if (!done)
	{
		GList *children = GTK_BOX(ui->run_vbox)->children;
		GList *l;
		GtkWidget * x = NULL;
		for (l=children; l!=NULL && !x; l=l->next)
		{
			GtkBoxChild *child = (GtkBoxChild*) l->data;
			GtkBin *frame = GTK_BIN(child->widget);
			GtkBin *eventbox = GTK_BIN(frame->child);
			StatusItemView *v = STATUS_ITEM_VIEW(eventbox->child);

			if (v->item == STATUS_ITEM(task))
				x = GTK_WIDGET(frame);
		}
		if (x!=NULL)
		{
			gtk_container_remove (GTK_CONTAINER(ui->run_vbox), x);
			done = TRUE;
		}
	}

	/* check the failure list */
	if (!done) {
		i = gtk_clist_find_row_from_data (GTK_CLIST(ui->fail_clist), task);
		if (i!=-1) {
			gtk_clist_remove (GTK_CLIST(ui->fail_clist), i);
			done = TRUE;
		}
	}

	pan_unlock ();

	pan_warn_if_fail (done!=FALSE);
}

static int
task_added_cb (gpointer obj, gpointer arg, gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	Task *task = TASK(obj);
	QueueTaskStatus status = queue_get_task_status (task);
	const int index = tm_index (GPOINTER_TO_INT(arg), status);

	/* add the task */
	if (status == QUEUE_TASK_STATUS_QUEUED || status == QUEUE_TASK_STATUS_CONNECTING)
		add_task_to_queue_clist (ui, task, index);
	else if (status == QUEUE_TASK_STATUS_RUNNING)
		add_task_to_run_clist (ui, task, index);
	else if (status==QUEUE_TASK_STATUS_FAILED || status==QUEUE_TASK_STATUS_ABORTING)
		add_task_to_fail_clist (ui, task, index);

	return 0;
}

static void
ui_populate (ManagerUI *ui)
{
	int i;
	const GSList* l = NULL;
	for (i=0, l=queue_get_tasks(); l!=NULL; l=l->next, ++i)
		task_added_cb (l->data, GINT_TO_POINTER(i), ui);
}

static int
task_removed_cb (gpointer obj, gpointer arg, gpointer user_data)
{
	remove_task ((ManagerUI*)user_data, TASK(obj));
	return 0;
}
static int
task_status_cb (gpointer obj, gpointer arg, gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	Task *task = TASK(obj);
	const int index = g_slist_index ((GSList*)queue_get_tasks(), task);

	remove_task (ui, task);

	task_added_cb (task, GINT_TO_POINTER(index), ui);

	return 0;
}
static int
task_moved_cb (gpointer obj, gpointer arg, gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	Task *task = TASK(obj);
	QueueTaskStatus status;
	int* pi = (int*)arg;
       
	status = queue_get_task_status (task);
	if (status==QUEUE_TASK_STATUS_QUEUED || status==QUEUE_TASK_STATUS_CONNECTING)
	{
		GtkCList* clist = GTK_CLIST(ui->queue_clist);
		Task *t = TASK(gtk_clist_get_row_data(clist,pi[0]));
		if (t==task) /* avoid feedback loop caused by drag from ui */
		{
			const gboolean b = ui->dampen_move_feedback_loop;
			ui->dampen_move_feedback_loop = TRUE;
			pan_lock();
			gtk_clist_row_move (clist,
				tm_index(pi[0], status),
				tm_index(pi[1], status));
			pan_unlock();
			ui->dampen_move_feedback_loop = b;
		}
	}
	else g_message ("FIXME");

	return 0;
}

#if 0
static int
index_of_nth_of_status (int n, QueueTaskStatus status)
{
	int i;
	const GSList* l;

	pthread_mutex_lock (&qlock);
	l = queue_get_tasks ();
	for (i=0; l!=NULL; ++i, l=l->next)
	{
		if (queue_get_task_status(TASK(l->data)) == status)
			--n;
		if (n<0)
			break;
	}
	pthread_mutex_unlock (&qlock);

	return l==NULL ? -1 : i;
}
#endif


static void
cancel_clicked_cb (GtkButton* button, gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	GtkCList *clist = GTK_CLIST(ui->queue_clist);
        GList *l;
	for (l=clist->selection; l!=NULL; l=l->next)
	{
		const int index = GPOINTER_TO_INT (l->data);
		Task * const task = gtk_clist_get_row_data (clist, index);
		queue_task_remove (task);
	}
}

static void
up_clicked_cb (GtkButton* button, gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	GtkCList *clist = GTK_CLIST(ui->queue_clist);
        GList *l;
	int last_index = -1;
	for (l=clist->selection; l!=NULL; l=l->next)
	{
		const int index = GPOINTER_TO_INT (l->data);
		Task * const task = gtk_clist_get_row_data (clist, index);
		int new_index = MAX (last_index+1, index-1);
		queue_task_move (task, new_index);
		last_index = new_index;
	}
}

static void
down_clicked_cb (GtkButton* button, gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	GtkCList *clist = GTK_CLIST(ui->queue_clist);
        GList *l;
	for (l=g_list_last(clist->selection); l!=NULL; l=l->prev)
	{
		const int index = GPOINTER_TO_INT (l->data);
		Task * const task = gtk_clist_get_row_data (clist, index);
		queue_task_move (task, index+1);
	}
}

static void
requeue_clicked_cb (GtkButton* button, gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	GtkCList *clist = GTK_CLIST(ui->fail_clist);
        GList *l;
	for (l=clist->selection; l!=NULL; l=l->next)
	{
		const int index = GPOINTER_TO_INT (l->data);
		Task * const task = gtk_clist_get_row_data (clist, index);
		queue_task_requeue_failed (task, -1);
	}
}


static void
queue_row_move_cb (
	GtkCList* clist,
	int old_index,
	int new_index,
	gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	Task *task = TASK(gtk_clist_get_row_data (clist, old_index));
	if (!ui->dampen_move_feedback_loop)
		queue_task_move (task, new_index);
}

static int
update_socket_lb_cb (gpointer user_data)
{
	char buf[128];
	int sockets = 0;
	int threads = 0;
	double kbps = 0.0;
	GList *l;
	ManagerUI *ui = (ManagerUI*) user_data;

	pan_lock();
	for (l=GTK_BOX(ui->run_vbox)->children; l!=NULL; l=l->next)
	{
		GtkBoxChild *child = (GtkBoxChild*) l->data;
		GtkBin *frame = GTK_BIN(child->widget);
		GtkBin *eventbox = GTK_BIN(frame->child);
		StatusItemView *v = STATUS_ITEM_VIEW(eventbox->child);

		++threads;
		if (v->sock != NULL)
		{
			++sockets;
			kbps += pan_socket_get_xfer_rate_kbps (v->sock);
		}
	}
	sprintf (buf, _("%d threads running, %d connections, total transfer speed is %.2f kb/s"), threads, sockets, kbps);
	gtk_label_set_text (GTK_LABEL(ui->socket_lb), buf);
	pan_unlock();

	return 1;
}

static void
dedicate_socket_cb (
	GtkToggleButton* tb,
	gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	if (!ui->dampen_connection_feedback_loop)
		queue_set_dedicated_article_thread (
			gtk_toggle_button_get_active (tb));
}

static void
max_connections_value_changed_cb (GtkAdjustment* adj, gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	if (!ui->dampen_connection_feedback_loop)
		queue_set_max_sockets ((int)adj->value);
}

static int
online_status_changed_cb (
	gpointer call_obj,
	gpointer call_arg,
	gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	int *a = (int*)call_arg;
	const gboolean online = a[0]!=0;
	gboolean dampen;
	GtkWidget * w;

	pan_lock();
	dampen = ui->dampen_connection_feedback_loop;
	ui->dampen_connection_feedback_loop = TRUE;

	/* online */
	w = ui->online_button;
	gtk_container_remove (GTK_CONTAINER(w), GTK_BIN(w)->child);
	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (
			online?traffic_green_xpm:traffic_red_xpm));
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, online ?
		_("Pan is Online") :
		_("Pan is Offline; no new tasks will processed"), "");
	gtk_widget_show_all (w);

	/* # of connections */
	w = ui->max_connections_sb;
	if (gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w)) != a[1])
		gtk_spin_button_set_value (GTK_SPIN_BUTTON(w), (gfloat)a[1] );

	/* dedicate a socket */
	w = ui->dedicate_read_socket_tb;
	if ((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))!=0)!=(a[2]!=0))
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), a[2]!=0);

	ui->dampen_connection_feedback_loop = FALSE;
	pan_unlock();

	return 0;
}

static int
max_tries_changed_cb (
	gpointer call_obj,
	gpointer call_arg,
	gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;
	gfloat value = (gfloat) GPOINTER_TO_INT(call_arg);

	pan_lock ();
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(ui->max_tries_sb), value);
	pan_unlock ();

	return 0;
}

static void
ui_destroy_cb (GtkObject* o, gpointer user_data)
{
	ManagerUI *ui = (ManagerUI*) user_data;

	g_source_remove (ui->timeout_id);

	pan_callback_remove (queue_task_added, task_added_cb, ui);
	pan_callback_remove (queue_task_removed, task_removed_cb, ui);
	pan_callback_remove (queue_task_status_changed, task_status_cb, ui);
	pan_callback_remove (queue_task_moved, task_moved_cb, ui);
	pan_callback_remove (queue_online_status_changed, online_status_changed_cb, ui);
	pan_callback_remove (queue_max_tries_changed, max_tries_changed_cb, ui);

	pan_warn_if_fail (ui == manager_ui);
	g_free (manager_ui);
	manager_ui = NULL;
}

static void
online_clicked_cb (GtkButton *b, gpointer user_data)
{
	queue_set_online (!queue_is_online());
}

static void
clear_queue_clicked_cb (GtkButton *b, gpointer user_data)
{
	GSList * queue = g_slist_copy ((GSList*)queue_get_tasks());
	GSList * l = NULL;

	for (l=queue; l!=NULL; l=l->next) {
		Task * task = TASK(l->data);
		if (queue_get_task_status(task) == QUEUE_TASK_STATUS_QUEUED)
			queue_task_remove(task);
	}

	g_slist_free (queue);
}

static ManagerUI*
task_manager_build_ui (void)
{
        GtkWidget* notebook;
        GtkWidget* vbox;
        GtkWidget* hbox;
        GtkWidget* w;
        GtkWidget* queued_top;
	GtkAdjustment* adj;
	ManagerUI* ui = NULL;
	const gboolean online = queue_is_online ();
	gchar* fail_clist_titles[2];
	gchar* queue_clist_titles[3];

	ui = g_new0 (ManagerUI, 1);

        notebook = gtk_notebook_new ();

        /* "queued" tab */
	queued_top = gtk_vbox_new (FALSE, 0);
	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);

	/* online button */
	w = ui->online_button = gtk_button_new ();
	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, online ?
		_("Pan is Online") :
		_("Pan is Offline; no new tasks will processed"), "");
	gtk_container_add (GTK_CONTAINER(w),
		gnome_pixmap_new_from_xpm_d (
			online?traffic_green_xpm:traffic_red_xpm));
	gtk_signal_connect (GTK_OBJECT(w), "clicked", online_clicked_cb, ui);
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);

	/* separator */
	w = gtk_vseparator_new ();
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, GNOME_PAD);

	/* up button */
	w = gtk_button_new ();
	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
	gtk_signal_connect (GTK_OBJECT(w), "clicked", up_clicked_cb, ui);
	gtk_container_add (
		GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (arrow_up_xpm));
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
		_("Move Selected Task(s) Up"), "");
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);

	/* down button */
	w = gtk_button_new ();
	gtk_signal_connect (GTK_OBJECT(w), "clicked", down_clicked_cb, ui);
	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (arrow_down_xpm));
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
		_("Move Selected Task(s) Down"), "");
	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);

#if 0
	/* separator */
	w = gtk_vseparator_new ();
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, GNOME_PAD);

	/* load button */
	w = gtk_button_new ();
	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Load Queue"), "");
	gtk_container_add (
		GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (queue_load_xpm));
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);

	/* save button */
	w = gtk_button_new ();
	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Save Queue"), "");
	gtk_container_add (
		GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (queue_save_xpm));
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
#endif

	/* separator */
	w = gtk_vseparator_new ();
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, GNOME_PAD);

	/* cancel button */
	w = gtk_button_new ();
	gtk_signal_connect (GTK_OBJECT(w), "clicked", cancel_clicked_cb, ui);
	gtk_container_add (
		GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (delete_row_xpm));
	gtk_tooltips_set_tip (
		GTK_TOOLTIPS(ttips), w, _("Cancel Selected Task(s)"), "");
	gtk_button_set_relief (GTK_BUTTON(w),GTK_RELIEF_NONE);
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);

	/* clear queue */
	w = gtk_button_new ();
	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Clear Queue"), "");
	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (delete_sheet_xpm));
	gtk_signal_connect (GTK_OBJECT(w), "clicked", clear_queue_clicked_cb, ui);
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);

	/* add button bar to the queued window */
	gtk_box_pack_start (GTK_BOX(queued_top), hbox, FALSE, FALSE, 0);

	/* queue list */
	queue_clist_titles[0] = _("Tries");
	queue_clist_titles[1] = _("Status");
	queue_clist_titles[2] = _("Task");
	ui->queue_clist = w = gtk_clist_new_with_titles (3, queue_clist_titles);
	gtk_clist_set_reorderable (GTK_CLIST(w), TRUE);
	gtk_clist_set_column_auto_resize (GTK_CLIST(w), 1, TRUE);
	gtk_clist_set_column_auto_resize (GTK_CLIST(w), 2, TRUE);
	gtk_clist_set_column_auto_resize (GTK_CLIST(w), 3, TRUE);
	gtk_clist_set_selection_mode (GTK_CLIST(w), GTK_SELECTION_EXTENDED);
	gtk_signal_connect (GTK_OBJECT(w), "row_move", queue_row_move_cb, ui);
	w = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER (w), ui->queue_clist);
	gtk_box_pack_start (GTK_BOX(queued_top), w, TRUE, TRUE, 0);

	/* "queued" rhs buttonbar */
	w = gtk_label_new (_("Queue"));
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), queued_top, w);

	/* "running" tab */
	vbox = gtk_vbox_new (FALSE, 3);
	gtk_container_set_border_width (GTK_CONTAINER(vbox), GNOME_PAD);

	/* max connections total */
	hbox = gtk_hbox_new (FALSE, 3);
	adj = GTK_ADJUSTMENT(gtk_adjustment_new (3, 0, 16, 1, 1, 0));
	w = ui->max_connections_sb = gtk_spin_button_new (adj, 1.0, 0);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(w), queue_get_max_sockets());
	gtk_signal_connect (
		GTK_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(w))),
		"value-changed",
		max_connections_value_changed_cb, ui);
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
	w = gtk_label_new (_("max connections total"));
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);

	/* edit per-server maximum */
	w = gtk_button_new ();
	gtk_container_add (
		GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (server_xpm));
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Set Per-Server Connection Limits"), "");
	gtk_signal_connect (
		GTK_OBJECT(w), "clicked", prefs_spawn_to_news_servers, NULL);
	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
	gtk_box_pack_end (GTK_BOX(hbox), w, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);


	/* dedicate read-article socket */
	w = ui->dedicate_read_socket_tb = gtk_check_button_new_with_label (
		_("Dedicate one connection to only read articles"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON(w), 
		queue_has_dedicated_article_thread());
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
		_("So that you can always read articles while other "
		  "connections handle downloading binaries, grabbing "
		  "new headers, or group lists."), "");
	gtk_signal_connect (GTK_OBJECT(w), "toggled", dedicate_socket_cb, ui);
	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);

	/* separator */
	gtk_box_pack_start (GTK_BOX(vbox), gtk_hseparator_new(), 0, 0, GNOME_PAD);


	/* socket information */
	ui->socket_lb = gtk_label_new (_("0 threads running, 0 connections, total transfer speed is 0.00 kb/s"));
	gtk_misc_set_alignment (GTK_MISC(ui->socket_lb), 0.0, 0.5);
	gtk_box_pack_start (GTK_BOX(vbox), ui->socket_lb, FALSE, FALSE, 0);
	ui->timeout_id = gtk_timeout_add (1000, update_socket_lb_cb, ui);

	/* running vbox */
	ui->run_vbox = gtk_vbox_new (FALSE, 0);
	w = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_add_with_viewport(
		GTK_SCROLLED_WINDOW(w), ui->run_vbox);
	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
	w = gtk_label_new (_("Running"));
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, w);

	/* "failed" tab */
	hbox = gtk_hbox_new (FALSE, 3);
	fail_clist_titles[0] = _("Task");
	fail_clist_titles[1] = _("Error");
	ui->fail_clist = w = gtk_clist_new_with_titles (2, fail_clist_titles);
	gtk_clist_set_column_auto_resize (GTK_CLIST(w), 0, TRUE);
	gtk_clist_set_column_auto_resize (GTK_CLIST(w), 1, TRUE);
	gtk_clist_set_selection_mode (GTK_CLIST(w), GTK_SELECTION_EXTENDED);
	w = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER (w), ui->fail_clist);
	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);

	/* requeue button */
	vbox = gtk_vbox_new (FALSE, 0);
	w = gtk_button_new ();
	gtk_container_add (
		GTK_CONTAINER(w),
		gnome_pixmap_new_from_xpm_d (requeue_xpm));
	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Requeue Task"), "");
	gtk_signal_connect (GTK_OBJECT(w), "clicked", requeue_clicked_cb, ui);
	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);

	/* remove task */
	w = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER(w),gnome_pixmap_new_from_xpm_d(x_xpm));
	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Cancel Task"), "");
	gtk_box_pack_end (GTK_BOX(vbox), w, FALSE, FALSE, 0);

	/* failed wrap-up */
	gtk_box_pack_end (GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
	w = gtk_label_new (_("Failures"));
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), hbox, w);

	ui->top = notebook;
	gtk_signal_connect (GTK_OBJECT(ui->top), "destroy", ui_destroy_cb, ui);

	pan_callback_add (queue_task_added, task_added_cb, ui);
	pan_callback_add (queue_task_removed, task_removed_cb, ui);
	pan_callback_add (queue_task_status_changed, task_status_cb, ui);
	pan_callback_add (queue_task_moved, task_moved_cb, ui);
	pan_callback_add (queue_online_status_changed, online_status_changed_cb, ui);
	pan_callback_add (queue_max_tries_changed, max_tries_changed_cb, ui);

	return ui;
}




/**
***  Titlebar
**/

static int
update_titlebar_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
{
	int running, queued, failed;
	const GSList* l;
	gchar* pch;

	running = queued = failed = 0;
	for (l=queue_get_tasks(); l!=NULL; l=l->next) {
		QueueTaskStatus status = queue_get_task_status (TASK(l->data));
		if (status==QUEUE_TASK_STATUS_QUEUED)
			++queued;
		else if (status==QUEUE_TASK_STATUS_RUNNING)
			++running;
		else if (status==QUEUE_TASK_STATUS_ABORTING || status==QUEUE_TASK_STATUS_FAILED)
			++failed;
	}

	if (failed)
		pch = g_strdup_printf ("Pan %s Task Manager (%d Queued, %d Running, %d Failed)", VERSION, queued, running, failed);
	else if (queued || running)
		pch = g_strdup_printf ("Pan %s Task Manager (%d Queued, %d Running)", VERSION, queued, running);
	else 
		pch = g_strdup_printf ("Pan %s Task Manager (Empty)", VERSION);
	pan_lock ();
	gtk_window_set_title (GTK_WINDOW(user_data), pch);
	pan_unlock ();
	g_free (pch);
	
	return 0;
}
static void
remove_update_titlebar_callbacks_cb (GtkObject* o, gpointer user_data)
{
	pan_callback_remove (queue_task_added, update_titlebar_cb, user_data);
	pan_callback_remove (queue_task_removed, update_titlebar_cb, user_data);
	pan_callback_remove (queue_task_status_changed, update_titlebar_cb, user_data);
}


	
	
void
task_manager_spawn (void)
{
	if (manager_ui != NULL)
	{
		if (!GTK_WIDGET_MAPPED(GTK_WIDGET(manager_ui->window)))
			gtk_widget_map ( GTK_WIDGET(manager_ui->window));
		gdk_window_raise (manager_ui->window->window);
		gdk_window_show (manager_ui->window->window);
		return;
	}

	if (!running_task_popup_menu) {
		running_task_popup_menu = gnome_popup_menu_new (running_task_popup);
		gtk_object_ref (GTK_OBJECT(running_task_popup_menu));
	}

	manager_ui = task_manager_build_ui ();
	ui_populate (manager_ui);
	manager_ui->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

	pan_callback_add (queue_task_added, update_titlebar_cb, manager_ui->window);
	pan_callback_add (queue_task_removed, update_titlebar_cb, manager_ui->window);
	pan_callback_add (queue_task_status_changed, update_titlebar_cb, manager_ui->window);
	gtk_signal_connect (GTK_OBJECT(manager_ui->window), "destroy", remove_update_titlebar_callbacks_cb, manager_ui->window);

	gtk_window_set_title (GTK_WINDOW(manager_ui->window), _("Pan - Task Manager"));
	gtk_container_add (GTK_CONTAINER(manager_ui->window), manager_ui->top);
	gtk_window_set_default_size (GTK_WINDOW (manager_ui->window), 300, 400);
	gtk_widget_show_all (manager_ui->window);
}

/************
*************  PRIVATE ROUTINES
************/

