/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*********************
**********************  Includes
*********************/

#include <config.h>

#include <string.h>

#include <glib.h>
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-i18n.h>

#include "debug.h"
#include "task.h"

/*********************
**********************  Defines / Enumerated types
*********************/

/*********************
**********************  Macros
*********************/

/*********************
**********************  Structures / Typedefs
*********************/

/*********************
**********************  Private Function Prototypes
*********************/

/*********************
**********************  Variables
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

/*********************
**********************  BEGINNING OF SOURCE
*********************/

/************
*************  PUBLIC ROUTINES
************/

/*****
******
*****/

void
task_constructor (Task *item,
                  PanObjectDestructor destructor,
                  StatusItemDescribeFunc describe,
                  TaskRunFunc run,
                  Server *sdata,
                  gboolean high_priority,
                  gboolean needs_socket)
{
	status_item_constructor (STATUS_ITEM(item), destructor, describe);

	item->sdata = sdata;
	item->hint_abort = FALSE;
	item->high_priority = high_priority;
	item->thread_id = 0;
	item->needs_socket = needs_socket;
	item->sock = NULL;
	item->is_leech = FALSE;
	item->is_article_reader = FALSE;
	item->tries = 0;
	item->run = run;

        debug (DEBUG_PAN_OBJECT, "task constructor: %p", item);
}

void
task_destructor (PanObject* item)
{
        debug (DEBUG_PAN_OBJECT, "task destructor: %p", item);
	status_item_destructor (PAN_OBJECT(item));
}

/*****
******
*****/

gint
task_run (Task *item)
{
	gint retval = 0;

	if (!item)
		retval = -1;
	else
	{
        	retval = (*item->run)(item);
		status_item_emit_done (STATUS_ITEM(item), retval);
	}

	return retval;
}

/*****
******
*****/

gboolean
task_is_high_priority (const Task *item)
{
	g_return_val_if_fail (item!=NULL, FALSE);

	return item->high_priority;
}

/*****
******
*****/

gboolean
task_needs_socket (const Task *item)
{
	g_return_val_if_fail (item!=NULL, FALSE);

	return item->needs_socket;
}

PanSocket*
task_get_socket (Task *item)
{
	g_return_val_if_fail (item!=NULL, NULL);

	return item->sock;
}

void
task_set_socket (Task *item, PanSocket *sock)
{
	g_return_if_fail (item!=NULL);
	g_return_if_fail (sock!=NULL);

	item->sock = sock;
}

extern const gchar* sockread_err_msg;
extern const gchar* sockwrite_err_msg;

void
task_emit_sockread_err (Task *item)
{
	status_item_emit_error (STATUS_ITEM(item), sockread_err_msg);
}

void
task_emit_sockwrite_err (Task *item)
{
	status_item_emit_error (STATUS_ITEM(item), sockwrite_err_msg);
}

/**
***
**/

void
task_hint_abort (Task* task, gboolean abort)
{
	g_return_if_fail (task!=NULL);

	task->hint_abort = abort;
}

