/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __TASK__H__
#define __TASK__H__

#include <pthread.h>

#include <glib.h>

#include "server.h"
#include "sockets.h"
#include "status-item.h"

#define TASK(a) ((Task*)a)

typedef struct _Task Task;
typedef int (*TaskRunFunc)(Task*);

struct _Task
{
	/* Parent Class */
	StatusItem status;

	/* Private Fields */  
	Server *sdata;
	PanSocket *sock;
	gboolean hint_abort;
	gboolean is_leech : 1;
	gboolean is_article_reader : 1;
	gboolean high_priority : 1;
	gboolean needs_socket : 1;
	int tries;
	pthread_t thread_id;

	/* virtual functions */
	TaskRunFunc run;
};

/**
***  PROTECTED
**/

void task_destructor      (PanObject*);

void task_constructor     (Task*,
                           PanObjectDestructor,
                           StatusItemDescribeFunc,
                           TaskRunFunc,
                           Server*,
                           gboolean high_priority,
                           gboolean needs_socket);

/**
***  PUBLIC
**/

gboolean task_is_high_priority    (const Task*);

gboolean task_needs_socket        (const Task*);

PanSocket* task_get_socket        (Task*);

void task_set_socket              (Task*,
                                   PanSocket*);

int task_run                      (Task*);

void task_emit_sockread_err       (Task*);

void task_emit_sockwrite_err      (Task*);

void task_hint_abort              (Task*,
                                   gboolean do_abort);

#endif
